﻿using Connected.Common.Workspace.Messaging;
using Connected.Entities.Consistency;
using Connected.Security.Identity;

namespace Connected.Common.Workspace;
/// <summary>
/// Specifies the read state of the <see cref="IWorkspaceEntity"/> entity.
/// </summary>
public enum ReadStatus
{
	/// <summary>
	/// The entity has not been read by the <see cref="IWorkspaceEntity.IdentityId"/>.
	/// </summary>
	New = 0,
	/// <summary>
	/// The entity has been read by the <see cref="IWorkspaceEntity.IdentityId"/>.
	/// </summary>
	Read = 1
}
/// <summary>
/// Specifies what kind of actions can be performed by the identity on each <see cref="IWorkspaceEntity"/> entity.
/// </summary>
[Flags]
public enum IdentityVerbs
{
	/// <summary>
	/// The entity is read only. Identity cannot perform any actions on it 
	/// except reading.
	/// </summary>
	ReadOnly = 0,
	/// <summary>
	/// The entity can be deleted from the workspace by the identity.
	/// </summary>
	AllowDelete = 1
}
/// <summary>
/// Represents a base entity for all workspace related entities.
/// </summary>
/// <remarks>
/// Digital Workspace offers many different services serving different purposes. Each service introduces
/// its own set of entities. All top level entities inherits from this entity, for example <see cref="IMessage"/>.
/// </remarks>
public interface IWorkspaceEntity : IConsistentEntity<long>
{
	/// <summary>
	/// The type of identity to which workspace entity belongs to (like Recipient).
	/// </summary>
	/// <remarks>
	/// The most common type of the identity entity is <see cref="IUser"/>.
	/// </remarks>
	string IdentityEntity { get; init; }
	/// <summary>
	/// The id of the identity to which workspace entity belongs. This could be the id of the user,
	/// device, organization unit or any other entity id. 
	/// </summary>
	string IdentityId { get; init; }
	/// <summary>
	/// The date and time the workspace entity has been created.
	/// </summary>
	/// <remarks>
	/// This value can represent the value when the origin entity has been created or when
	/// the workspace entity has been created. This value is specific to the service implementation.
	/// </remarks>
	DateTimeOffset Created { get; init; }
	/// <summary>
	/// The date and time the workspace entity expires.
	/// </summary>
	/// <remarks>
	/// Expired workspace entities are automatically deleted from the system. If this value is <c>null</c>,
	/// workspace entity never expires.
	/// </remarks>
	DateTimeOffset? Expires { get; init; }
	/// <summary>
	/// The title of the entity.
	/// </summary>
	/// <remarks>
	/// This should be as short as possible so user can quickly recognize the nature of the entity.
	/// </remarks>
	string Title { get; init; }
	/// <summary>
	/// The short description of the entity. Can be <c>null</c>.
	/// </summary>
	string? Description { get; init; }
	/// <summary>
	/// The read status of the entity. This serves primary for a UI purpose to enable visual feedback
	/// to the user.
	/// </summary>
	ReadStatus Status { get; init; }
	/// <summary>
	/// The domain to which this entity belongs.
	/// </summary>
	/// <remarks>
	/// Each workspace entity should have one and only one domain to which it originates.
	/// </remarks>
	string Domain { get; init; }
	/// <summary>
	/// Zero or more tags that represents additional meta data of the workspace entity.
	/// </summary>
	string? Tags { get; init; }
	/// <summary>
	/// The entity which triggered this workspace entity to be created. Can be <c>null</c> if it was
	/// automatically generated by the underlying system.
	/// </summary>
	string? AuthorEntity { get; init; }
	/// <summary>
	/// The id of the entity which created this workspace entity. Can be <c>null</c> if it was
	/// automatically generated by the underlying system.
	/// </summary>
	string? AuthorId { get; init; }
	/// <summary>
	/// Actions that can be performed by the identity on the workspace entity.
	/// </summary>
	IdentityVerbs Verbs { get; init; }
}
